/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var canvas;
var context;
var imgdimx;
var imgdimy;
var sliderSP;
var xSP;
var xS;
var segment;
var variableA;
var variableB;
var variableC;
var TS;
var T_SP;
var S_SP;
var pointT;
var pointJ;
var pointS;
var pointS1;
var pointSP;
var branche;
var valeurMin;
var valeurMax;


function init() {
    canvas = document.getElementById('dessin');
    var largeur_fenetre = 'innerWidth' in window ? window.innerWidth : document.documentElement.offsetWidth;
    var hauteur_fenetre = 'innerHeight' in window ? window.innerHeight : document.documentElement.offsetHeight;
    imgdimx = Math.min(largeur_fenetre - 350, hauteur_fenetre - 50);
    if (imgdimx <= 0)
        imgdimx = 500;
    imgdimy = imgdimx;
    //imgdimx = 800;
    //imgdimy = 600;
    canvas.width = imgdimx;
    canvas.height = imgdimy;
    context = canvas.getContext('2d');
    context.translate(imgdimx/2 + 0.5, imgdimy/2 + 0.5);
    context.font = 'bold 12px sans-serif';
    branche = 0;
    xS = -150;
    
    variableA = 100;
    variableB = 60;
    
    valeurMin = -imgdimx/2;
    valeurMax = Math.floor(xS);
    xSP = (valeurMin+valeurMax) / 2;
    sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
    
    variableC = Math.sqrt((variableA*variableA)+(variableB*variableB));
    
    pointT = new Point2D(-variableC, 0);
    pointJ = new Point2D(variableC, 0);
    
    majCroquis();
    
    segment = document.getElementById('segment');
    if (segment.addEventListener)
        segment.addEventListener('change', changerCurseur, false);
    else if (segment.attachEvent)
        segment.attachEvent('onchange', changerCurseur);
}

function changerCurseur() {
    if(segment.selectedIndex == 0) {
        valeurMin = -imgdimx/2;
        valeurMax = Math.floor(xS);
        xSP = (valeurMin+valeurMax) / 2;
        sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
        branche = 0;
        majCroquis();
    }
    if(segment.selectedIndex == 1) {
        valeurMin = -imgdimx/2;
        valeurMax = Math.floor(xS)-1;
        xSP = (valeurMin+valeurMax) / 2;
        sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
        branche = 1;
        majCroquis();
    }
    if(segment.selectedIndex == 2) {
        valeurMin = -imgdimx/2;
        valeurMax = Math.floor(xS)-1;
        xSP = (valeurMin+valeurMax) / 2;
        sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
        branche = 2;
        majCroquis();
    }
    if(segment.selectedIndex == 3) {
        valeurMin = variableA;
        valeurMax = imgdimx/2;
        xSP = (valeurMin+valeurMax) / 2;
        sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
        branche = 3;
        majCroquis();
    }
    if(segment.selectedIndex == 4) {
        valeurMin = variableA;
        valeurMax = imgdimx/2;
        xSP = (valeurMin+valeurMax) / 2;
        sliderSP = new Curseur('curseurSP', valeurMin, valeurMax, xSP, 97, false, modifCurseur);
        branche = 4;
        majCroquis();
    }
}


function modifCurseur(e) {
    var id, valeur;
    id = e.target.id;
    if (id == 'curseurSP') {
        valeur = sliderSP.getValeur();
        xSP = valeur;
    }
    majCroquis();
}

function majCroquis() {
    context.fillStyle = "white";
    context.fillRect(-imgdimx/2, -imgdimy/2, imgdimx, imgdimy);
    
    ligneHorizontal();
    afficherPoint(pointT, "black", "T");
    afficherPoint(pointJ, "black", "J");
    asymptote();
    hyperbole();
    
    pointS = new Point2D(xS, -h(xS));
    pointS1 = new Point2D(xS, h(xS));
    afficherPoint(pointS, "#E31818", "S");
    afficherPoint(pointS1, "#E31818", "S1");
    
    if (branche == 1 || branche == 3)
        pointSP = new Point2D(xSP, -h(xSP));
    else
        pointSP = new Point2D(xSP, h(xSP));
    
    S_SP = distance(pointS, pointSP);
    TS = distance(pointT, pointS);
    T_SP = distance(pointT, pointSP);
    if (branche != 0) {
        afficherPoint(pointSP, "blue", "S'");
        if (branche == 1 || branche == 2)
            courbe1();
        else
            courbe2();
    }
}


// Tracs
function ligneHorizontal() {
    context.beginPath();
    context.strokeStyle = "lightGray";
    context.moveTo(-imgdimx/2, 0.0);
    context.lineTo(imgdimx/2, 0.0);
    context.stroke();
}

function asymptote() {
    context.beginPath();
    context.strokeStyle = "lightGray";
    context.moveTo(-imgdimx, -droite(imgdimx));
    context.lineTo(imgdimx, droite(imgdimx));
    context.moveTo(-imgdimx, droite(imgdimx));
    context.lineTo(imgdimx, -droite(imgdimx));
    context.stroke();
}

function hyperbole() {
    var precision, mu1, mu2, x, y;
    // quation paramtrique x =  a * cosh(mu), y = b * sinh(mu)
    precision = 20;
    mu1 = -Math.floor(Math.ceil(precision * acosh((imgdimx/2)/variableA)));
    mu2 = -mu1;
    context.beginPath();
    context.strokeStyle = "black";
    // branche entourant J
    for (mu = mu1; mu <= mu2; mu++) {
        x = variableA * cosh(mu / precision);
        y = variableB * sinh(mu / precision);
        if (mu == mu1)
            context.moveTo(x, y);
        else
            context.lineTo(x, y);
    }
    // branche entourant T
    for (mu = mu1; mu <= mu2; mu++) {
        x = - variableA * cosh(mu/precision);
        y = variableB * sinh(mu/precision);
        if (mu == mu1)
            context.moveTo(x, y);
        else
            context.lineTo(x, y);
    }
    context.stroke();
}

function courbe1() {
    var thetaSSP, a, e, b, mx, my, mu1, mu2, x, y;
    thetaSSP = Math.atan2(pointSP.y - pointS.y, pointSP.x - pointS.x);
    a = (T_SP - TS) / 2;
    e = S_SP / (T_SP - TS);
    //avec quation paramtrique pour l'hyperbole droite + rotation
    mx = (pointS.x + pointSP.x) / 2;
    my = (pointS.y + pointSP.y) / 2;
    context.save();
    context.translate(mx, my);
    context.rotate(thetaSSP);
    context.translate(-mx, -my);
    b = Math.sqrt(Math.pow(a, 2) * (Math.pow(e, 2) - 1));
    precision = 10;
    mu1 = -Math.floor(Math.ceil(precision * acosh((imgdimx - pointS.x)/a)));
    mu2 = -mu1;
    context.beginPath();
    context.strokeStyle = "blue";
    // branche entourant S'
    for (mu = mu1; mu <= mu2; mu++) {
        x = a * cosh(mu / precision) + mx;
        y = b * sinh(mu / precision) + my;
        if (mu == mu1)
            context.moveTo(x, y);
        else
            context.lineTo(x, y);
    }
    // branche entourant S
    for (mu = mu1; mu <= mu2; mu++) {
        x = - a * cosh(mu/precision) + mx;
        y = b * sinh(mu/precision) + my;
        if (mu == mu1)
            context.moveTo(x, y);
        else
            context.lineTo(x, y);
    }
    context.stroke();
    context.restore();
}

function courbe2() {
    // dessin d'une ellipse droite de la bonne taille centre au milieu de SS', puis
    // rotation d'angle SS' par rapport  ce milieu
    var thetaSSP, a, e, b, mx, my;
    thetaSSP = Math.atan2(pointSP.y - pointS.y, pointSP.x - pointS.x);
    a = (T_SP + TS) / 2;
    e = S_SP / (T_SP + TS);
    b = a * Math.sqrt(1 - Math.pow(e, 2));
    mx = (pointS.x + pointSP.x) / 2;
    my = (pointS.y + pointSP.y) / 2;
    context.save();
    context.translate(mx, my);
    context.rotate(thetaSSP);
    context.translate(-mx, -my);
    context.beginPath();
    context.strokeStyle = "blue";
    dessinerEllipse(context, mx - a, my - b, 2*a, 2*b);
    context.stroke();
    context.restore();
}

function dessinerEllipse(ctx, x, y, w, h)  {
    var kappa, ox, oy, xe, ye, xm, ym;
    kappa = .5522848;
    ox = (w / 2) * kappa;
    oy = (h / 2) * kappa;
    xe = x + w;
    ye = y + h;
    xm = x + w / 2;
    ym = y + h / 2;

    ctx.beginPath();
    ctx.moveTo(x, ym);
    ctx.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);
    ctx.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);
    ctx.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);
    ctx.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);
    ctx.closePath();
    ctx.stroke();
}


function afficherPoint(pt, couleur, titre) {
    context.beginPath();
    context.fillStyle = couleur;
    context.arc(pt.x, pt.y, 1, 0, 2 * Math.PI, true);
    context.fillText(titre, Math.floor(pt.x) + 2, Math.floor(pt.y) -2);
    context.fill();
}


function Point2D(x, y) {
    this.x = x;
    this.y = y;
}

function distance(point1, point2) {
    var xd, yd;
    xd = 0;
    yd = 0;
    xd = point2.x - point1.x;
    xd = xd * xd;
    yd = point2.y - point1.y;
    yd = yd * yd;
      
    return Math.sqrt(xd + yd);
}


// Equations
function droite(x) {
    return(-(variableB/variableA)*x);
}

function h(x) {
    return(variableB * Math.sqrt(((x*x) / (variableA*variableA)) - 1));
}


function acosh(x) {
    return (Math.log(x + Math.sqrt(x + 1) * Math.sqrt(x - 1)));
}

function cosh(x) {
    return (Math.pow(Math.E, x)+Math.pow(Math.E, -x))/2;
}

function sinh(x) {
    return (Math.pow(Math.E, x)-Math.pow(Math.E, -x))/2;
}

